@@ -1,4 +1,4 @@
-/* $Id: global.c 4379 2009-02-16 23:06:09Z astyanax $ */
+/* $Id: global.c 4395 2009-07-12 03:36:58Z astyanax $ */
 /**************************************************************************
  *   global.c                                                             *
  *                                                                        *
@@ -36,6 +36,9 @@ sigjmp_buf jump_buf;
 bool jump_buf_main = FALSE;
 	/* Have we set jump_buf so that we return to main() after a
 	 * SIGWINCH? */
+bool use_undo = FALSE;
+	/* Are we actually using the undo code - disabled by default
+	   as the undo code is too unstable */
 #endif
 
 #ifndef DISABLE_WRAPJUSTIFY
@@ -232,7 +235,7 @@ void add_to_funcs(short func, int menus, const char *desc, const char *help,
     subnfunc *f;
 
     if (allfuncs == NULL) {
-	allfuncs = nmalloc(sizeof(subnfunc));
+	allfuncs = (subnfunc *) nmalloc(sizeof(subnfunc));
 	f = allfuncs;
     } else {
 	for (f = allfuncs; f->next != NULL; f = f->next)
@@ -292,7 +295,7 @@ void add_to_sclist(int menu, char *scstring, short func, int toggle, int execute
     sc *s;
 
     if (sclist == NULL) {
-	sclist = nmalloc(sizeof(sc));
+	sclist = (sc *) nmalloc(sizeof(sc));
 	s = sclist;
         s->next = NULL;
     } else {
@@ -778,11 +781,13 @@ void shortcut_init(bool unjustify)
     add_to_funcs(DO_UNINDENT, MMAIN, N_("Unindent Text"),
 	IFSCHELP(nano_unindent_msg), FALSE, NOVIEW);
 
-    add_to_funcs(DO_UNDO, MMAIN, N_("Undo"),
-	IFSCHELP(nano_undo_msg), FALSE, NOVIEW);
+    if (use_undo) {
+	add_to_funcs(DO_UNDO, MMAIN, N_("Undo"),
+	    IFSCHELP(nano_undo_msg), FALSE, NOVIEW);
 
-    add_to_funcs(DO_REDO, MMAIN, N_("Redo"),
-	IFSCHELP(nano_redo_msg), TRUE, NOVIEW);
+	add_to_funcs(DO_REDO, MMAIN, N_("Redo"),
+	    IFSCHELP(nano_redo_msg), TRUE, NOVIEW);
+    }
 
 #endif
 
@@ -1064,8 +1069,10 @@ void shortcut_init(bool unjustify)
     add_to_sclist(MMAIN, "M-6", DO_COPY_TEXT, 0, TRUE);
     add_to_sclist(MMAIN, "M-}", DO_INDENT_VOID, 0, TRUE);
     add_to_sclist(MMAIN, "M-{", DO_UNINDENT, 0, TRUE);
-    add_to_sclist(MMAIN, "M-U", DO_UNDO, 0, TRUE);
-    add_to_sclist(MMAIN, "M-E", DO_REDO, 0, TRUE);
+    if (use_undo) {
+	add_to_sclist(MMAIN, "M-U", DO_UNDO, 0, TRUE);
+ 	add_to_sclist(MMAIN, "M-E", DO_REDO, 0, TRUE);
+    }
     add_to_sclist(MALL, "^F", DO_RIGHT, 0, TRUE);
     add_to_sclist(MALL, "^B", DO_LEFT, 0, TRUE);
     add_to_sclist(MMAIN, "^Space", DO_NEXT_WORD_VOID, 0, TRUE);
@@ -1214,7 +1221,7 @@ void iso_me_harder_funcmap(short func)
     else if (func == DO_RIGHT)
 	do_right();
     else if (func == DO_ENTER)
-	do_enter();
+	do_enter(FALSE);
     else if (func == DO_EXIT)
 	do_exit();
     else if (func == DO_FIRST_LINE)
