@@ -1,4 +1,4 @@
-/* $Id: nano.c 4381 2009-02-17 03:05:17Z astyanax $ */ 
+/* $Id: nano.c 4395 2009-07-12 03:36:58Z astyanax $ */ 
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
@@ -891,6 +891,10 @@ void usage(void)
 #endif
     print_opt("-t", "--tempfile",
 	N_("Auto save on exit, don't prompt"));
+#ifndef NANO_TINY
+    print_opt("-u", "--undo", N_("Allow generic undo [EXPERIMENTAL]"));
+#endif
+
     print_opt("-v", "--view", N_("View mode (read-only)"));
 #ifndef DISABLE_WRAPPING
     print_opt("-w", "--nowrap", N_("Don't wrap long lines"));
@@ -1853,7 +1857,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 		output[i] = '\n';
 	    /* Newline to Enter, if needed. */
 	    else if (output[i] == '\n') {
-		do_enter();
+		do_enter(FALSE);
 		i++;
 		continue;
 	    }
@@ -1909,7 +1913,7 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 	if (!ISSET(NO_WRAP)) {
 	    bool do_refresh_save = do_refresh;
 
-	    do_refresh = do_wrap(openfile->current);
+	    do_refresh = do_wrap(openfile->current, FALSE);
 
 	    /* If we needed to call edit_refresh() before this, we'll
 	     * still need to after this. */
@@ -2012,6 +2016,7 @@ int main(int argc, char **argv)
 	{"noconvert", 0, NULL, 'N'},
 	{"smooth", 0, NULL, 'S'},
 	{"quickblank", 0, NULL, 'U'},
+	{"undo", 0, NULL, 'u'},
 	{"wordbounds", 0, NULL, 'W'},
 	{"autoindent", 0, NULL, 'i'},
 	{"cut", 0, NULL, 'k'},
@@ -2053,11 +2058,11 @@ int main(int argc, char **argv)
     while ((optchr =
 #ifdef HAVE_GETOPT_LONG
 	getopt_long(argc, argv,
-		"h?ABC:DEFHIKLNOQ:RST:UVWY:abcdefgijklmo:pqr:s:tvwxz",
+		"h?ABC:DEFHIKLNOQ:RST:UVWY:abcdefgijklmo:pqr:s:tuvwxz",
 		long_options, NULL)
 #else
 	getopt(argc, argv,
-		"h?ABC:DEFHIKLNOQ:RST:UVWY:abcdefgijklmo:pqr:s:tvwxz")
+		"h?ABC:DEFHIKLNOQ:RST:UVWY:abcdefgijklmo:pqr:s:tuvwxz")
 #endif
 		) != -1) {
 	switch (optchr) {
@@ -2206,6 +2211,11 @@ int main(int argc, char **argv)
 	    case 't':
 		SET(TEMP_FILE);
 		break;
+#ifndef NANO_TINY
+	    case 'u':
+		use_undo = TRUE;
+		break;
+#endif
 	    case 'v':
 		SET(VIEW_MODE);
 		break;
