@@ -1,4 +1,4 @@
-/* $Id: nano.h 4374 2009-02-09 04:03:20Z astyanax $ */
+/* $Id: nano.h 4391 2009-04-21 05:34:08Z astyanax $ */
 /**************************************************************************
  *   nano.h                                                               *
  *                                                                        *
@@ -28,6 +28,12 @@
 #include <config.h>
 #endif
 
+#ifdef NEED_XOPEN_SOURCE_EXTENDED
+#ifndef _XOPEN_SOURCE_EXTENDED
+#define _XOPEN_SOURCE_EXTENDED 1
+#endif /* _XOPEN_SOURCE_EXTENDED */
+#endif /* NEED_XOPEN_SOURCE_EXTENDED */
+
 #ifdef __TANDEM
 /* Tandem NonStop Kernel support. */
 #include <floss.h>
@@ -44,6 +50,10 @@
 #include <sys/param.h>
 #endif
 
+#ifdef HAVE_STDARG_H
+#include <stdarg.h>
+#endif
+
 /* Macros for flags. */
 #define SET(bit) flags |= bit
 #define UNSET(bit) flags &= ~bit
@@ -170,7 +180,7 @@ typedef enum {
 }  function_type;
 
 typedef enum {
-    ADD, DEL, REPLACE, SPLIT, UNSPLIT, CUT, UNCUT, INSERT, OTHER
+    ADD, DEL, REPLACE, SPLIT, UNSPLIT, CUT, UNCUT, ENTER, INSERT, OTHER
 } undo_type;
 
 #ifdef ENABLE_COLOR
@@ -282,6 +292,8 @@ typedef struct undo {
 	/* Where did this  action begin or end */
     char *strdata;
 	/* String type data we will use for ccopying the affected line back */
+    char *strdata2;
+	/* Sigh, need this too it looks like */
     int xflags;
 	/* Some flag data we need */
 
@@ -720,6 +732,7 @@ typedef struct subnfunc {
 /* Extra bits for the undo function */
 #define UNDO_DEL_DEL		(1<<0)
 #define UNDO_DEL_BACKSPACE	(1<<1)
+#define UNDO_SPLIT_MADENEW	(1<<2)
 
 /* Since in ISO C you can't pass around function pointers anymore,
   let's make some integer macros for function names, and then I
