@@ -1,4 +1,4 @@
-/* $Id: proto.h 4378 2009-02-16 21:04:00Z astyanax $ */
+/* $Id: proto.h 4395 2009-07-12 03:36:58Z astyanax $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
@@ -30,6 +30,7 @@
 #ifndef NANO_TINY
 extern sigjmp_buf jump_buf;
 extern bool jump_buf_main;
+extern bool use_undo;
 #endif
 
 #ifndef DISABLE_WRAPJUSTIFY
@@ -76,7 +77,7 @@ extern char *quoteerr;
 extern size_t quotelen;
 #endif
 #endif
-bool nodelay_mode;
+extern bool nodelay_mode;
 extern char *answer;
 
 extern ssize_t tabsize;
@@ -624,14 +625,14 @@ void do_unindent(void);
 void do_undo(void);
 void do_redo(void);
 #endif
-void do_enter(void);
+void do_enter(bool undoing);
 #ifndef NANO_TINY
 RETSIGTYPE cancel_command(int signal);
 bool execute_command(const char *command);
 #endif
 #ifndef DISABLE_WRAPPING
 void wrap_reset(void);
-bool do_wrap(filestruct *line);
+bool do_wrap(filestruct *line, bool undoing);
 #endif
 #if !defined(DISABLE_HELP) || !defined(DISABLE_WRAPJUSTIFY)
 ssize_t break_line(const char *line, ssize_t goal
@@ -811,11 +812,6 @@ const char *mac_format_msg;
 const char *append_msg;
 const char *prepend_msg;
 const char *backup_file_msg;
-const char *dos_format_msg;
-const char *mac_format_msg;
-const char *append_msg;
-const char *prepend_msg;
-const char *backup_file_msg;
 const char *gototext_msg;
 const char *new_buffer_msg;
 void iso_me_harder_funcmap(short func);
