@@ -1,4 +1,4 @@
-/* $Id: rcfile.c 4375 2009-02-11 05:45:57Z astyanax $ */
+/* $Id: rcfile.c 4396 2009-07-12 06:13:36Z astyanax $ */
 /**************************************************************************
  *   rcfile.c                                                             *
  *                                                                        *
@@ -89,6 +89,7 @@ static const rcoption rcopts[] = {
     {"smarthome", SMART_HOME},
     {"smooth", SMOOTH_SCROLL},
     {"tabstospaces", TABS_TO_SPACES},
+    {"undo", 0},
     {"whitespace", 0},
     {"wordbounds", WORD_BOUNDS},
 #endif
@@ -416,6 +417,7 @@ void parse_keybinding(char *ptr)
 	return;
     }
 
+    menu = strtomenu(menuptr);
     newsc = strtosc(menu, funcptr);
     if (newsc == NULL) {
 	rcfile_error(
@@ -423,7 +425,6 @@ void parse_keybinding(char *ptr)
 	return;
     }
 
-    menu = strtomenu(menuptr);
     if (menu < 1) {
 	rcfile_error(
 		N_("Could not map name \"%s\" to a menu"), menuptr);
@@ -908,6 +909,15 @@ void parse_rcfile(FILE *rcstream
 	option = ptr;
 	ptr = parse_next_word(ptr);
 
+#ifndef NANO_TINY
+	/* FIXME: Hack which should go away ASAP */
+	if (strcasecmp(option,  "undo") == 0) {
+	    use_undo = TRUE;
+	    shortcut_init(0);
+	    continue;
+	}
+#endif
+
 	for (i = 0; rcopts[i].name != NULL; i++) {
 	    if (strcasecmp(option, rcopts[i].name) == 0) {
 #ifdef DEBUG
