@@ -1,4 +1,4 @@
-/* $Id: text.c 4379 2009-02-16 23:06:09Z astyanax $ */ 
+/* $Id: text.c 4397 2009-07-27 04:16:44Z astyanax $ */ 
 /**************************************************************************
  *   text.c                                                               *
  *                                                                        *
@@ -360,21 +360,25 @@ void do_unindent(void)
 /* undo a cut, or re-do an uncut */
 void undo_cut(undo *u)
 {
-	cutbuffer = copy_filestruct(u->cutbuffer);
+    /* If we cut the magicline may was well not crash :/ */
+    if (!u->cutbuffer)
+	return;
 
-        /* Compute cutbottom for the uncut using out copy */
-	for (cutbottom = cutbuffer; cutbottom->next != NULL; cutbottom = cutbottom->next)
-	    ;
+    cutbuffer = copy_filestruct(u->cutbuffer);
 
-	/* Get to where we need to uncut from */
-	if (u->mark_set && u->mark_begin_lineno < u->lineno)
-	    do_gotolinecolumn(u->mark_begin_lineno, u->mark_begin_x+1, FALSE, FALSE, FALSE, FALSE);
-	else
-	    do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+    /* Compute cutbottom for the uncut using out copy */
+    for (cutbottom = cutbuffer; cutbottom->next != NULL; cutbottom = cutbottom->next)
+	;
 
-        copy_from_filestruct(cutbuffer, cutbottom);
-	free_filestruct(cutbuffer);
-	cutbuffer = NULL;
+    /* Get to where we need to uncut from */
+    if (u->mark_set && u->mark_begin_lineno < u->lineno)
+	do_gotolinecolumn(u->mark_begin_lineno, u->mark_begin_x+1, FALSE, FALSE, FALSE, FALSE);
+    else
+	do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+
+    copy_from_filestruct(cutbuffer, cutbottom);
+    free_filestruct(cutbuffer);
+    cutbuffer = NULL;
 
 }
 
@@ -383,40 +387,44 @@ void redo_cut(undo *u) {
     int i;
     filestruct *t, *c;
 
-	do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
-	openfile->mark_set = u->mark_set;
-	if (cutbuffer)
-	    free(cutbuffer);
-	cutbuffer = NULL;
+    /* If we cut the magicline may was well not crash :/ */
+    if (!u->cutbuffer)
+	return;
 
-	/* Move ahead the same # lines we had if a marked cut */
-	if (u->mark_set) {
-	    for (i = 1, t = openfile->fileage; i != u->mark_begin_lineno; i++)
-		t = t->next;
-	    openfile->mark_begin = t;
-	} else if (!u->to_end) {
-	    /* Here we have a regular old potentially multi-line ^K cut.  We'll
-		need to trick nano into thinking it's a marked cut to cut more
-		than one line again */
-	    for (c = u->cutbuffer, t = openfile->current; c->next != NULL && t->next != NULL; ) {
+    do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+    openfile->mark_set = u->mark_set;
+    if (cutbuffer)
+	free(cutbuffer);
+    cutbuffer = NULL;
+
+    /* Move ahead the same # lines we had if a marked cut */
+    if (u->mark_set) {
+	for (i = 1, t = openfile->fileage; i != u->mark_begin_lineno; i++)
+	    t = t->next;
+	openfile->mark_begin = t;
+    } else if (!u->to_end) {
+	/* Here we have a regular old potentially multi-line ^K cut.  We'll
+	   need to trick nano into thinking it's a marked cut to cut more
+	   than one line again */
+	for (c = u->cutbuffer, t = openfile->current; c->next != NULL && t->next != NULL; ) {
 
 #ifdef DEBUG
-	    fprintf(stderr, "Advancing, lineno  = %d, data = \"%s\"\n", t->lineno, t->data);
+	fprintf(stderr, "Advancing, lineno  = %d, data = \"%s\"\n", t->lineno, t->data);
 #endif
-		c = c->next;
-		t = t->next;
-	    }
-	    openfile->mark_begin = t;
-	    openfile->mark_begin_x = 0;
-	    openfile->mark_set = TRUE;
-	}
+	    c = c->next;
+	    t = t->next;
+	 }
+	openfile->mark_begin = t;
+ 	openfile->mark_begin_x = 0;
+	openfile->mark_set = TRUE;
+    }
 
-	openfile->mark_begin_x = u->mark_begin_x;
-	do_cut_text(FALSE, u->to_end, TRUE);
-	openfile->mark_set = FALSE;
-        openfile->mark_begin = NULL;
-        openfile->mark_begin_x = 0;
-	edit_refresh_needed = TRUE;
+    openfile->mark_begin_x = u->mark_begin_x;
+    do_cut_text(FALSE, u->to_end, TRUE);
+    openfile->mark_set = FALSE;
+    openfile->mark_begin = NULL;
+    openfile->mark_begin_x = 0;
+    edit_refresh_needed = TRUE;
 }
 
 /* Undo the last thing(s) we did */
@@ -474,13 +482,15 @@ void do_undo(void)
 	    openfile->current_x += strlen(u->strdata);
 	break;
     case SPLIT:
-	undidmsg = _("line split");
-	free(f->data);
-	f->data = mallocstrcpy(NULL, u->strdata);
-	if (f->next != NULL) {
-	    filestruct *tmp = f->next;
-	    unlink_node(tmp);
-	    delete_node(tmp);
+	undidmsg = _("line wrap");
+	f->data = nrealloc(f->data, strlen(f->data) + strlen(u->strdata) + 1);
+	strcpy(&f->data[strlen(f->data) - 1], u->strdata);
+	if (u->strdata2 != NULL)
+	    f->next->data = mallocstrcpy(f->next->data, u->strdata2);
+	else {
+	    filestruct *foo = openfile->current->next;
+	    unlink_node(foo);
+	    delete_node(foo);
 	}
 	renumber(f);
 	break;
@@ -503,6 +513,16 @@ void do_undo(void)
 	undidmsg = _("text uncut");
 	redo_cut(u);
 	break;
+    case ENTER:
+	undidmsg = _("line break");
+	if (f->next) {
+	    filestruct *foo = f->next;
+	    f->data = nrealloc(f->data, strlen(f->data) + strlen(f->next->data) + 1);
+	    strcat(f->data,  f->next->data);
+	    unlink_node(foo);
+	    delete_node(foo);
+	}
+	break;
     case INSERT:
 	undidmsg = _("text insert");
 	cutbuffer = NULL;
@@ -527,11 +547,13 @@ void do_undo(void)
 	u->strdata = f->data;
 	f->data = data;
 	break;
+
     default:
 	undidmsg = _("Internal error: unknown type.  Please save your work");
 	break;
 
     }
+    renumber(f);
     do_gotolinecolumn(u->lineno, u->begin, FALSE, FALSE, FALSE, TRUE);
     statusbar(_("Undid action (%s)"), undidmsg);
     openfile->current_undo = openfile->current_undo->next;
@@ -541,7 +563,7 @@ void do_undo(void)
 void do_redo(void)
 {
     undo *u = openfile->undotop;
-    filestruct *f = openfile->current, *t;
+    filestruct *f = openfile->current;
     int len = 0;
     char *undidmsg, *data;
 
@@ -591,15 +613,16 @@ void do_redo(void)
 	free(f->data);
 	f->data = data;
 	break;
+    case ENTER:
+	undidmsg = _("line break");
+	do_gotolinecolumn(u->lineno, u->begin+1, FALSE, FALSE, FALSE, FALSE);
+	do_enter(TRUE);
+	break;
     case SPLIT:
-	undidmsg = _("line split");
-	t = make_new_node(f);
-	t->data = mallocstrcpy(NULL, &u->strdata[u->begin]);
-	data = mallocstrncpy(NULL, f->data, u->begin);
-	data[u->begin] = '\0';
-	free(f->data);
-	f->data = data;
-	splice_node(f, t, f->next);
+	undidmsg = _("line wrap");
+	if (u->xflags & UNDO_SPLIT_MADENEW)
+	    prepend_wrap = TRUE;
+        do_wrap(f, TRUE);
 	renumber(f);
 	break;
     case UNSPLIT:
@@ -652,7 +675,7 @@ void do_redo(void)
 #endif /* !NANO_TINY */
 
 /* Someone hits Enter *gasp!* */
-void do_enter(void)
+void do_enter(bool undoing)
 {
     filestruct *newnode = make_new_node(openfile->current);
     size_t extra = 0;
@@ -660,7 +683,8 @@ void do_enter(void)
     assert(openfile->current != NULL && openfile->current->data != NULL);
 
 #ifndef NANO_TINY
-    update_undo(SPLIT);
+    if (!undoing)
+	add_undo(ENTER);
 
 
     /* Do auto-indenting, like the neolithic Turbo Pascal editor. */
@@ -811,6 +835,10 @@ void add_undo(undo_type current_action)
     char *data;
     openfilestruct *fs = openfile;
     static undo *last_cutu = NULL; /* Last thing we cut to set up the undo for uncut */
+    ssize_t wrap_loc;	/* For calculating split beginning */
+
+    if (!use_undo)
+	return;
 
     /* Ugh, if we were called while cutting not-to-end, non-marked and on the same lineno,
        we need to  abort here */
@@ -839,6 +867,7 @@ void add_undo(undo_type current_action)
     fs->undotop = u;
     fs->current_undo = u;
     u->strdata = NULL;
+    u->strdata2 = NULL;
     u->cutbuffer = NULL;
     u->cutbottom  = NULL;
     u->mark_set = 0;
@@ -871,8 +900,20 @@ void add_undo(undo_type current_action)
 	    u->strdata = data;
 	}
 	break;
-    case INSERT:
     case SPLIT:
+	wrap_loc = break_line(openfile->current->data, fill
+#ifndef DISABLE_HELP
+	, FALSE
+#endif
+	);
+	u->strdata = mallocstrcpy(NULL, &openfile->current->data[wrap_loc]);
+	/* Don't both saving the next line if we're not prepending as a new line
+	   will be created */
+	if (prepend_wrap)
+	    u->strdata2 = mallocstrcpy(NULL, fs->current->next->data);
+	u->begin = wrap_loc;
+	break;
+    case INSERT:
     case REPLACE:
 	data = mallocstrcpy(NULL, fs->current->data);
 	u->strdata = data;
@@ -894,6 +935,8 @@ void add_undo(undo_type current_action)
 	    u->cutbottom = last_cutu->cutbottom;
 	}
 	break;
+    case ENTER:
+	break;
     case OTHER:
 	statusbar(_("Internal error: unknown type.  Please save your work."));
 	break;
@@ -902,7 +945,7 @@ void add_undo(undo_type current_action)
 #ifdef DEBUG
     fprintf(stderr, "fs->current->data = \"%s\", current_x = %d, u->begin = %d, type = %d\n",
 			fs->current->data,  fs->current_x, u->begin, current_action);
-    fprintf(stderr, "left update_add...\n");
+    fprintf(stderr, "left add_undo...\n");
 #endif
     fs->last_action = current_action;
 }
@@ -919,6 +962,9 @@ void update_undo(undo_type action)
     int len = 0;
     openfilestruct *fs = openfile;
 
+    if (!use_undo)
+	return;
+
 #ifdef DEBUG
         fprintf(stderr, "action = %d, fs->last_action = %d,  openfile->current->lineno = %d",
 		action, fs->last_action, openfile->current->lineno);
@@ -931,7 +977,7 @@ void update_undo(undo_type action)
     /* Change to an add if we're not using the same undo struct
        that we should be using */
     if (action != fs->last_action
-	|| (action != CUT && action != INSERT
+	|| (action != CUT && action != INSERT && action != SPLIT
 	    && openfile->current->lineno != fs->current_undo->lineno)) {
         add_undo(action);
 	return;
@@ -1011,9 +1057,15 @@ void update_undo(undo_type action)
 	break;
     case INSERT:
 	u->mark_begin_lineno = openfile->current->lineno;
+	break;
     case SPLIT:
+	/* This will only be called if we made a completely new line,
+	   and as such we should note that so we can destroy it later */
+	u->xflags = UNDO_SPLIT_MADENEW;
+	break;
     case UNSPLIT:
 	/* These cases are handled by the earlier check for a new line and action */
+    case ENTER:
     case OTHER:
 	break;
     }
@@ -1043,7 +1095,7 @@ void wrap_reset(void)
 /* We wrap the given line.  Precondition: we assume the cursor has been
  * moved forward since the last typed character.  Return TRUE if we
  * wrapped, and FALSE otherwise. */
-bool do_wrap(filestruct *line)
+bool do_wrap(filestruct *line, bool undoing)
 {
     size_t line_len;
 	/* The length of the line we wrap. */
@@ -1108,6 +1160,9 @@ bool do_wrap(filestruct *line)
 	return FALSE;
 
 #ifndef NANO_TINY
+    if (!undoing)
+	add_undo(SPLIT);
+
     /* If autoindent is turned on, and we're on the character just after
      * the indentation, we don't wrap. */
     if (ISSET(AUTOINDENT)) {
@@ -1202,6 +1257,8 @@ bool do_wrap(filestruct *line)
     null_at(&line->data, wrap_loc);
 
     if (prepending) {
+	if (!undoing)
+	    update_undo(SPLIT);
 	/* If we're prepending, copy the text from the next line, minus
 	 * the indentation that we already copied above. */
 	strcat(new_line, next_line);
